#include "ParamOption.h"

namespace ffgl
{

ParamOption::ParamOption( const std::string& name, FFUInt32 defaultIndex ) :
	ParamOption( name, name, defaultIndex )
{
	// Note: use AddOption() to add to options vector after this constructor called
}

ParamOption::ParamOption( const std::string& name, const std::string& uniformName, FFUInt32 defaultIndex ) :
	ParamFloat( name, ParameterType::FF_TYPE_OPTION, uniformName, static_cast< float >( defaultIndex ) )
{
	// Note: use AddOption() to add to options vector after this constructor called
}

ParamOption::ParamOption( const std::string& name, std::vector< Option > options, FFUInt32 defaultIndex ) :
	ParamOption( name, name, std::move( options ), defaultIndex )
{
}

ParamOption::ParamOption( const std::string& name, const std::string& uniformName, std::vector< Option > options, FFUInt32 defaultIndex ) :
	ParamFloat( name, ParameterType::FF_TYPE_OPTION, uniformName, static_cast< float >( defaultIndex ) ),
	options( std::move( options ) )
{
}

ParamOption::~ParamOption()
{
}

const std::string& ParamOption::GetDisplay() const
{
	static std::string displayString;
	displayString = std::to_string( GetFloatValue() );
	return displayString;
}

FFResult ParamOption::SetMixedValue( const FFMixed mixed )
{
	// Error if index is negative
	if (mixed.FloatValue < 0.0f)
		return FF_FAIL;

	return SetIndex( (FFUInt32)mixed.FloatValue );
}

float ParamOption::GetFloatValue() const
{
	FFUInt32 currentIndex = GetIndex();

	// Beware of empty options vector!
	if( currentIndex >= options.size() )
		return 0.0f;

	return options[ currentIndex ].value;
}

void ParamOption::AddOption( Option option )
{
	options.push_back( option );
}

FFUInt32 ParamOption::GetNumOptions() const
{
	return (FFUInt32)options.size();
}

const ParamOption::Option* ParamOption::GetOption( FFUInt32 index ) const
{
	if( index >= options.size() )
		return nullptr;

	return &options[ index ];
}

const char* ParamOption::GetOptionName( FFUInt32 index ) const
{
	if( index >= options.size() )
		return nullptr;

	return options[ index ].name.c_str();
}

FFResult ParamOption::GetOptionValue( FFUInt32 index, float& outValue ) const
{
	if( index >= options.size() )
		return FF_FAIL;

	outValue = options[ index ].value;
	return FF_SUCCESS;
}

FFUInt32 ParamOption::GetDefaultIndex() const
{
	FFUInt32 defaultIndex = static_cast< FFUInt32 >( defaultValue );
	if( defaultIndex >= options.size() )
		defaultIndex = 0U;

	return defaultIndex;
}

FFUInt32 ParamOption::GetIndex() const
{
	FFUInt32 index = static_cast< FFUInt32 >( ParamFloat::GetFloatValue() );
	if( index >= options.size() )
		index = 0U;

	return index;
}

FFResult ParamOption::SetIndex( FFUInt32 index )
{
	if( index >= options.size() )
		return FF_FAIL;

	ParamFloat::SetFloatValue( static_cast< float >( index ) );
	return FF_SUCCESS;
}

bool ParamOption::IsCurrentOption( const std::string& optionName ) const
{
	FFUInt32 currentIndex = GetIndex();

	if( currentIndex >= options.size() )
		return false;

	return options[ currentIndex ].name.compare( optionName ) == 0;
}

}//End namespace ffgl